@@ -1,6 +1,6 @@
 /*
- * twclock:  A world clock implemented with openMotif widgets
- * Copyright (C) 1997 - 2006 Ted Williams WA0EIR
+ * twclockCB:  A world clock implemented with openMotif widgets
+ * Copyright (C) 1997 - 2007 Ted Williams WA0EIR
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge. MA 02139, USA.
  * See the COPYING file in this directory
  *
- * Version: 2.5 - Jan 2006
+ * Version: 2.7 - Aug 2007
  */
 
 /*
@@ -29,11 +29,10 @@
 #include <fcntl.h>
 
 #include "twclock.h"
-#include "cwlib.h"
 
-char timepath[100];
 Pixel saveColor;
 Boolean newTimeFlag;
+char timepath[100];
 
 /*
  * CATCH INPUT EVENTS
@@ -63,10 +62,10 @@ void input_event (Widget w, XtPointer client_data, XEvent *event, Boolean *cont)
  */
 void popup_cb(Widget w, XtPointer client_data, XtPointer cbs)
 {
-   int set_zone = (int) client_data;
+   int btn_num = (int) client_data;
    Widget zone_fsb;
 
-   switch (set_zone)
+   switch (btn_num)
    {
       case LOCAL_BTN:
          strcpy (timepath, "");
@@ -115,22 +114,50 @@ void popup_cb(Widget w, XtPointer client_data, XtPointer cbs)
          break;
 
       case ID_NOW_BTN:
-         cw_set_volume (appRes.cwLevel);
-         cw_set_send_speed (appRes.cwSpeed);
-         cw_send_string ((const unsigned char *) appRes.cwStr);
+         cw_send_str (appRes.cwStr);
          break;
-
+#if 0
+      case ABOUT_BTN:
+         printf ("Hello World # %d\n", btn_num);
+         break;
+#endif
       case QRT:
+         cleanup ();
          exit(0);
          break;
 
       default:
-         fprintf (stderr, "Error: invalid menu button\n");
+         fprintf (stderr, "Error 1: Invalid menu button\n");
          break;
    }
 }
  
 
+/*
+ * callback for About Cascade button
+ * switch on the button number
+ */
+void abt_cas_cb(Widget w, XtPointer client_data, XtPointer cbs)
+{
+   int btn = (int) client_data;
+
+   switch (btn)
+   {
+      case ABOUT_DIAG:
+         popup_about ();
+         break;
+
+      case HELP_DIAG:
+         popup_help ();
+         break;
+
+      default:
+         fprintf (stderr, "Error 2: Invalid button number");
+         break;
+   }
+}
+
+
 /*
  * GETTIME FUNCTION
  * A call to this function is hardcoded near the end of widget
@@ -240,21 +267,23 @@ void TBchangeCB(Widget w, XtPointer cdata, XtPointer cbs)
    {
       case True:
          flashFlag = True;
+         /* Save the bg color */
          XtVaGetValues (w,
-            XmNselectColor, &saveColor,
+            XmNbackground, &saveColor,
             NULL);
 
+         /* add a timeout for selected delay */
          alarm_id = XtAppAddTimeOut(app,
             (unsigned long) (((60 * appRes.minutes) + appRes.seconds) * 1000),
-            (XtTimerCallbackProc)timer,
+            (XtTimerCallbackProc)alarm_timer,
             NULL);
 
          break;
 
       case False:
-         /* Restore original selectColor */
+         /* Restore original background */
          XtVaSetValues (w,
-            XmNselectColor, saveColor,
+            XmNbackground, saveColor,
             NULL);
 
          /* Remove the alarm time out*/
@@ -265,18 +294,19 @@ void TBchangeCB(Widget w, XtPointer cdata, XtPointer cbs)
          {
             XtRemoveTimeOut (auto_id);
          }
+
          /* unregister and re-register the time out again */
          /* Why? - How does time_id get changed when the toggle */
          /* button is unset? This hack removes and adds a very */
          /* short time_id, which has no effect on the gui, but */
-         /* keeps timer_id valid. timer_id was moved from gettime */
-         /* twclock.h so it can be seen from here */
+         /* keeps timer_id valid. */
          XtRemoveTimeOut (time_id);
          time_id=XtAppAddTimeOut(app,
             1L,
             (XtTimerCallbackProc)gettime,
-            (XtPointer)NULL);
-               break;
+            NULL);
+         break;
+
       default:
          fprintf (stderr, "Error: invalid toggle button state\n");
          break;
@@ -285,12 +315,12 @@ void TBchangeCB(Widget w, XtPointer cdata, XtPointer cbs)
 
 /*
  * ALARM TIMER FUNCTION
- * First called when the ID timer times out.  It will cw id, blink the
+ * First called when the ID alarm_timer times out.  It will cw id, blink the
  * toggle button and beep as requested in the resource file.  It then
- * changes the timer delay to 1 sec and registers another timeout.
+ * changes the delay to 1 sec and registers another timeout.
  * Unsetting the toggle button unregisters the timeouts.
  */
-void timer (void)
+void alarm_timer (void)
 {
    FILE *con;
 
@@ -302,7 +332,7 @@ void timer (void)
       {
          alarm_id = XtAppAddTimeOut(app,
             (unsigned long) (((60 * appRes.minutes) + appRes.seconds) * 1000),
-            (XtTimerCallbackProc)timer,
+            (XtTimerCallbackProc)alarm_timer,
             (XtPointer)NULL);
       }
 
@@ -310,10 +340,10 @@ void timer (void)
       {
          /* restore bg color */
          XtVaSetValues (call_toggleB,
-            XmNselectColor, saveColor,
+            XmNbackground, saveColor,
             NULL);
 
-         /* reset flags and wait for next alarm timeout */
+         /* set up flags and wait for next alarm timeout */
          count = 0;
          cwIDdone = False;
          flashFlag = True;
@@ -326,45 +356,39 @@ void timer (void)
    }
 
    /* If we get here, popup the clock, in case its hidden 
-    * Send cw id,flash and blink if we should and then
+    * Send cw id, flash and blink (if we should) and then
     * set a one second timeout
     */
+   XtPopup (clock_shell, XtGrabNone);
 
-   if ( appRes.cwID == True && cwIDdone == False)
+   if (appRes.cwID == True && cwIDdone == False)
    {
-      cw_set_volume (appRes.cwLevel);
-      cw_set_send_speed (appRes.cwSpeed);
-      cw_send_string ((const unsigned char*) appRes.cwStr);
+      cw_send_str (appRes.cwStr);
       cwIDdone = True;
    }
 
-   XtPopup( clock_shell, XtGrabNone );
-
-   if ((con = fopen ("/dev/console", "w")) == 0)
+   if (appRes.beep == True)
    {
-      fprintf (stderr, "can't open console\n");
-   }
-   else
-   {
-      /* do beep? */
-      if (appRes.beep == True)
+      if ((con = fopen ("/dev/console", "w")) == 0)
       {
-         fprintf (con, "\a");
+         fprintf (stderr, "can't open console\n");
+         return;
       }
+      fprintf (con, "\a");
       fclose (con);
    }
 
    if (appRes.blink == True && flashFlag == True)
    {
       XtVaSetValues (call_toggleB,
-         XmNselectColor, appRes.flashColor,
+         XmNbackground, appRes.flashColor,
          NULL);
       flashFlag = False;
    }
    else
    {
       XtVaSetValues (call_toggleB,
-         XmNselectColor, saveColor,
+         XmNbackground, saveColor,
          NULL);
       flashFlag = True;
       
@@ -372,7 +396,7 @@ void timer (void)
    /* reset auto reset alarm for one second */
    auto_id=XtAppAddTimeOut(app,
       1000L,
-      (XtTimerCallbackProc)timer,
+      (XtTimerCallbackProc)alarm_timer,
       NULL);   
 }
 
@@ -387,9 +411,13 @@ void settimer (void)
    char minutes[10], seconds[10];
    static Widget settimerDiag = NULL;
    static Widget timeTFs[2];
-   Widget setForm, label1, frame1, timeForm, min_label, sec_label;
+   Widget label1, frame1, timeForm, min_label, sec_label;
    Widget label2, frame2, optForm, autoResetTB, cwidTB, beepTB, blinkTB;
-   Widget label3, frame3, cwForm, cwSpdScale, cwLvlScale;
+   Widget label3, frame3, cwRC, cwSpdScale, cwLvlScale;
+   Widget sep1, okPB, cancelPB;
+#if SCALE_BUG == 1
+   Widget label4, label5;
+#endif
    XmString xms;
 
    /* Convert the current minutes and seconds to strings */ 
@@ -399,12 +427,15 @@ void settimer (void)
    /* Lazy Instantiation */   
    if (settimerDiag != NULL)
    {
+#if 0
+     /* not needed with openMotif - maybe Lesstif does */
      /* reload times in case we closed via Cancel button */
      /* but got to remove the callbacks first */
       XtRemoveCallback (timeTFs[0], XmNmodifyVerifyCallback, timerModVerCB,
          (XtPointer) NULL);
       XtRemoveCallback (timeTFs[1], XmNmodifyVerifyCallback, timerModVerCB,
          (XtPointer) NULL);
+#endif
 
       XtVaSetValues (timeTFs[0],
          XmNvalue, minutes,
@@ -413,39 +444,34 @@ void settimer (void)
       XtVaSetValues (timeTFs[1],
          XmNvalue, seconds,
          NULL);
-
+#if 0
+      /* see above */
       /* now add the callbacks again */
       XtAddCallback (timeTFs[0], XmNmodifyVerifyCallback, timerModVerCB,
          (XtPointer) NULL);
       XtAddCallback (timeTFs[1], XmNmodifyVerifyCallback, timerModVerCB,
          (XtPointer) NULL);
-
+#endif
       XtManageChild (settimerDiag);
       return;
    }
 
    /* Create a message box dialog and set the dialog shell values */   
-   settimerDiag = XmCreateMessageDialog (clock_shell, "settimer",
-      (ArgList) NULL, 0);
-   
+   settimerDiag = XmCreateFormDialog (clock_shell, "settimerDiag", NULL, 0);
+
    XtVaSetValues (XtParent(settimerDiag),
       XmNtitle, "Twclock Options",
       NULL); 
 
    XtVaSetValues (settimerDiag,          
       XmNdialogStyle, XmDIALOG_MODELESS,
-      NULL);
-
-   /* Don't want a help button */
-   XtUnmanageChild (XmMessageBoxGetChild (settimerDiag,XmDIALOG_HELP_BUTTON));   
-   /* Add form to the message box workspace for goodies */
-   setForm = XtVaCreateWidget ("setForm", xmFormWidgetClass, settimerDiag,
-      XmNverticalSpacing, 5,
+      XmNverticalSpacing, 10,
+      XmNhorizontalSpacing, 10,
       NULL);
 
    xms = XmStringCreateLocalized ("Timer");
    label1 = XtVaCreateManagedWidget ("label1", xmLabelWidgetClass,
-      setForm,
+      settimerDiag,
       XmNlabelString, xms,
       XmNtopAttachment, XmATTACH_FORM,
       XmNleftAttachment, XmATTACH_FORM,
@@ -454,7 +480,7 @@ void settimer (void)
       NULL);
    XmStringFree(xms);
 
-   frame1 = XtVaCreateWidget ("frame1", xmFrameWidgetClass, setForm,
+   frame1 = XtVaCreateWidget ("frame1", xmFrameWidgetClass, settimerDiag,
       XmNshadowType, XmSHADOW_OUT,
       XmNshadowThickness, 3,
       XmNtopAttachment, XmATTACH_WIDGET,
@@ -510,7 +536,7 @@ void settimer (void)
 
    xms = XmStringCreateLocalized ("Alarms");
    label2 = XtVaCreateManagedWidget ("label2", xmLabelWidgetClass,
-      setForm,
+      settimerDiag,
       XmNlabelString, xms,
       XmNtopAttachment, XmATTACH_WIDGET,
       XmNtopWidget, timeForm,
@@ -520,7 +546,7 @@ void settimer (void)
       NULL);
    XmStringFree(xms);
 
-   frame2 = XtVaCreateWidget ("frame1", xmFrameWidgetClass, setForm,
+   frame2 = XtVaCreateWidget ("frame1", xmFrameWidgetClass, settimerDiag,
       XmNshadowType, XmSHADOW_OUT,
       XmNshadowThickness, 3,
       XmNtopAttachment, XmATTACH_WIDGET,
@@ -590,7 +616,7 @@ void settimer (void)
 
    xms = XmStringCreateLocalized ("CW ID");
    label3 = XtVaCreateManagedWidget ("label3", xmLabelWidgetClass,
-      setForm,
+      settimerDiag,
       XmNlabelString, xms,
       XmNtopAttachment, XmATTACH_WIDGET,
       XmNtopWidget, optForm,
@@ -600,7 +626,7 @@ void settimer (void)
       NULL);
    XmStringFree(xms);
 
-   frame3 = XtVaCreateWidget ("frame3", xmFrameWidgetClass, setForm,
+   frame3 = XtVaCreateWidget ("frame3", xmFrameWidgetClass, settimerDiag,
       XmNshadowType, XmSHADOW_OUT,
       XmNshadowThickness, 3,
       XmNmarginWidth, 5,
@@ -612,46 +638,79 @@ void settimer (void)
       XmNrightAttachment, XmATTACH_FORM,
       NULL);
 
-   cwForm = XtVaCreateWidget ("cwForm", xmFormWidgetClass, frame3,
+   cwRC = XtVaCreateWidget ("cwRC", xmRowColumnWidgetClass, frame3,
       NULL);
 
    xms = XmStringCreateLocalized ("CW Speed");
+   /* if we have the scale bug, put the sting in a label */
+#if SCALE_BUG == 1
+   label4 = XtVaCreateManagedWidget ("label3", xmLabelWidgetClass, cwRC,
+      XmNlabelString, xms,
+      NULL);
+#endif
+
    cwSpdScale = XtVaCreateManagedWidget ("cwSptScale", xmScaleWidgetClass,
-      cwForm,
-      XmNtitleString, xms,
+      cwRC,
       XmNscaleMultiple, 5,
       XmNorientation, XmHORIZONTAL,
       XmNshowValue, True,
       XmNminimum, 10,
       XmNmaximum, 40,
-      XmNtopAttachment, XmATTACH_FORM,
-      XmNleftAttachment, XmATTACH_FORM,
-      XmNbottomAttachment, XmATTACH_NONE,
-      XmNrightAttachment, XmATTACH_FORM,
+      XmNtitleString, xms,
       NULL);
+
+   /* free the string and sync cwSpdScale with cwSpeed resource */
    XmStringFree(xms);
-   /* sync cwSpdScale with cwSpeed resource */
    XmScaleSetValue (cwSpdScale, appRes.cwSpeed);
 
    xms = XmStringCreateLocalized ("CW Level");
+   /* if we have the scale bug, put the sting in a label */
+#if SCALE_BUG == 1
+   label5 = XtVaCreateManagedWidget ("label3", xmLabelWidgetClass, cwRC,
+      XmNlabelString, xms,
+      NULL);
+#endif
+
    cwLvlScale = XtVaCreateManagedWidget ("cwLvlScale", xmScaleWidgetClass,
-      cwForm,
-      XmNtitleString, xms,
+      cwRC,
       XmNorientation, XmHORIZONTAL,
       XmNshowValue, True,
+      XmNtitleString, xms,
+      NULL);
+
+   /* free the string and sync cwLvlScale with cwLevel resource */
+   XmStringFree(xms);
+   XmScaleSetValue (cwLvlScale, appRes.cwLevel);
+
+   sep1 = XtVaCreateManagedWidget ("sep1", xmSeparatorWidgetClass, settimerDiag,
       XmNtopAttachment, XmATTACH_WIDGET,
-      XmNtopWidget, cwSpdScale,
+      XmNtopWidget, frame3,
       XmNleftAttachment, XmATTACH_FORM,
       XmNbottomAttachment, XmATTACH_NONE,
       XmNrightAttachment, XmATTACH_FORM,
       NULL);
-   XmStringFree(xms);
-   /* sync cwLvlScale with cwLevel resource */
-   XmScaleSetValue (cwLvlScale, appRes.cwLevel);
+
+   okPB = XtVaCreateManagedWidget ("Ok", xmPushButtonWidgetClass, settimerDiag,
+      XmNtopAttachment, XmATTACH_WIDGET,
+      XmNtopWidget, sep1,
+      XmNleftAttachment, XmATTACH_FORM,
+      XmNbottomAttachment, XmATTACH_FORM,
+      XmNrightAttachment, XmATTACH_POSITION,
+      XmNrightPosition, 20,
+      NULL);
+
+   cancelPB = XtVaCreateManagedWidget ("Cancel", xmPushButtonWidgetClass,
+      settimerDiag,
+      XmNtopAttachment, XmATTACH_WIDGET,
+      XmNtopWidget, sep1,
+      XmNleftAttachment, XmATTACH_POSITION,
+      XmNleftPosition, 80,
+      XmNbottomAttachment, XmATTACH_FORM,
+      XmNrightAttachment, XmATTACH_FORM,
+      NULL);
 
    /* Add the callbacks and 2 modifyVerify */
-
-   XtAddCallback (settimerDiag, XmNokCallback, timerOkCB,
+   XtAddCallback (okPB, XmNactivateCallback, timerOkCB,
                (XtPointer) timeTFs);
 
    XtAddCallback (timeTFs[0], XmNmodifyVerifyCallback, timerModVerCB,
@@ -681,31 +740,31 @@ void settimer (void)
     * to the resourse that needs tweaking.
     */
    XtAddCallback (cwSpdScale, XmNdragCallback, scalesCB,
-               (XtPointer) &appRes.cwSpeed);
+               (XtPointer) 0);
+               //(XtPointer) &appRes.cwSpeed);
 
    XtAddCallback (cwSpdScale, XmNvalueChangedCallback, scalesCB,
-               (XtPointer) &appRes.cwSpeed);
+               (XtPointer) 0);
 
    XtAddCallback (cwLvlScale, XmNdragCallback, scalesCB,
-               (XtPointer) &appRes.cwLevel);
+               (XtPointer) 1);
 
    XtAddCallback (cwLvlScale, XmNvalueChangedCallback, scalesCB,
-               (XtPointer) &appRes.cwLevel);
+               (XtPointer) 1);
 
-   /* Pop up the timer dialog */
+   /* Manage all the dialog kids */
    XtManageChild (timeForm);
    XtManageChild (frame1);
    XtManageChild (optForm);
    XtManageChild (frame2);
-   XtManageChild (cwForm);
+   XtManageChild (cwRC);
    XtManageChild (frame3);
-   XtManageChild (setForm);
    XtManageChild (settimerDiag);
 }
 
 
 /*
- * Callbacks for setting the timer
+ * Callbacks for setting the alarm_timer
  */
 
 
@@ -741,7 +800,7 @@ void timerOkCB (Widget w, XtPointer client_data, XtPointer cbs)
       XtRemoveTimeOut (alarm_id);
       alarm_id = XtAppAddTimeOut(app,
          (unsigned long) (((60 * appRes.minutes) + appRes.seconds) * 1000),
-         (XtTimerCallbackProc)timer, (XtPointer)NULL);
+         (XtTimerCallbackProc)alarm_timer, (XtPointer)NULL);
    }
    /* finally, clear the flag */
    newTimeFlag = False;
@@ -792,15 +851,29 @@ void optionsCB (Widget w, XtPointer client_data, XtPointer cbs)
 /*
  * scalesCB drag/valueChanged callbacks for CW ID scales
  * maintains the values of CW ID option resources
- * client_data is a pointer to appRes where state is stored
+ * client_data is is stored
  */
 void scalesCB (Widget w, XtPointer client_data, XtPointer cbs)
 {
-   int *ptr = (int *)client_data;
+   int val;
+   int k = (int)client_data;
    XmScaleCallbackStruct *cbsptr = (XmScaleCallbackStruct *) cbs;
 
-   *ptr = cbsptr->value;
-   /* doesn't work...  cw_set_send_speed (appRes.cwSpeed); */
-   /* but this does */
-   cw_set_volume (appRes.cwLevel);  
+   val = cbsptr->value;
+
+   switch (k)
+   {
+      case 0:
+         appRes.cwSpeed = val;
+         cw_set_wpm (appRes.cwSpeed);
+         break;
+
+      case 1:
+         appRes.cwLevel = val;
+         cw_set_send_volume (appRes.cwLevel);  
+         break;
+
+      default:
+         fprintf (stderr, "scalesCB: invalid argument\n");
+   }
 }
